function S = QSspeczero(u,meth,pw)

% Estimate Multivariate Spectrum at Zero Using a QS window.
% The data are in u, in which rows correspond to observations and columns
% correspond to variables.
% If there are two input arguments and pw=1, then prewhiten before
% computing spectral density

if nargin == 3
    if pw == 1
        p=25;
        hasconstant=1;
        [beta,V,uu,vcovbeta]=estimatevar(u,p,hasconstant);
        u=uu;
    end
end

[nobs,k]=size(u);
if k > nobs
    error('fatal (QSspeczero) rows of u should correspond to number of observations')
end

u = u - repmat(mean(u),nobs,1);	% Remove sample mean

S=zeros(k,k);

if meth == 1
    w=ones(k,1);
    num=0;
    den=0;
    for ii = 1:k
        er=u(:,ii);
        [rhoa,sig2a]=stat(er);
        num=num+w(ii)*(4*rhoa^2*sig2a^2/(1-rhoa)^8);
        den=den+w(ii)*(sig2a^2/(1-rhoa)^4);
    end
    
    alph=num/den;
    band=1.3221*((alph*nobs)^(1/5));
    maxlags=nobs-1;
elseif meth == 2
    maxlags=nobs-1;
    band=150;
end
    
for lag = 0:maxlags
    
    Rho = u(1:nobs-lag,:)'*u(1+lag:nobs,:);	% Vectorized (fast!!)
    Rho = Rho/nobs;
    if lag >= 1, Rho = Rho + Rho'; end
    x = lag/band;
    
    if meth == 1
        
        term = 6*pi*x/5;
        
        if lag == 0
            wt = 1;
        else
            wt = 25*(sin(term)/term - cos(term))/(12*pi^2*x^2);
        end
        
    elseif meth == 2        
        
        if x < 0.5  
            wt = 1 - 6*x^2 + 6*x^3;
        elseif x < 1
            wt = 2*(1-x)^3; 
        else 
            wt = 0;
        end
        
    end
    
    S = S + wt*Rho;
    
end

%recolor the spectral density if there has been prewhitening
if nargin == 3
    if pw == 1
        bbet=beta(:,[2:end]);
        n=size(bbet,1);
        sum=eye(n);
        
        for ii = 1:p
            sum=sum-bbet(:,(ii-1)*n+[1:n]);
        end
        
        Isum=inv(sum);
        S=Isum*S*Isum';
    end
end
