function [loglik] = SARMA(xx,p,q,N,data,per)
%this takes as input a vector containing the values of the parameters of a
%SARMA model, and the data, and computes the associated Gaussian likelihood

if isempty(per) == 1
    spectrum=0;
else
    spectrum=1;
end

[sarcoef,macoef,v] = opt2coef(xx,p,q,N);

if min(eig(v)) < 0
    loglik=4.e12;
    return
end

%check the root conditions and if they are violated, return with a penalty
[ip] = checkroot(sarcoef,macoef,p,q);

if ip == 1
    loglik=1.e12;
    return
elseif ip == 2
    loglik=2.e12;
    return
elseif ip == 3
    loglik=3.e12;
    return
end

%set up the Kalman matrices
F=zeros((p+q+1)*N,(p+q+1)*N);
V=zeros((p+q+1)*N,(p+q+1)*N);
H=zeros(N,(p+q+1)*N);

I=eye(N);

for ii = 1:p
    F(1:N,(ii-1)*N+[1:N])=sarcoef(ii)*I;
end
F(1:N,p*N+[1:q*N])=macoef;
for ii = 2:p
    F((ii-1)*N+[1:N],(ii-2)*N+[1:N])=I;
end
for ii = 1:q
    F((p+1)*N+(ii-1)*N+[1:N],p*N+(ii-1)*N+[1:N])=I;
end

tt=p*N;
V([1:N],[1:N])=v;
V(tt+[1:N],[1:N])=v;
V([1:N],tt+[1:N])=v;
V(tt+[1:N],tt+[1:N])=v;

H(:,[1:N])=eye(N);



%get the likelihood function value
if spectrum == 0
    
    R=zeros(N,N);
    alf=zeros(N,1);
    xini=zeros((p+q+1)*N,1);
    [P] = uncondvar1(V,F);
    if min(diag(P)) < 0 | min(eig(P)) < 0
        loglik=5.e12;
        return
    end
    
    [loglik] = mykalman(data',F,H,V,R,xini,P,alf);
    
elseif spectrum == 1
    
    T=max(size(data));
    [loglik] = getspectff(T,F,H,V,per);
    
end

loglik=-loglik;
