function cov = bwspeczero(x,lt);
% Estimate Multivariate Spectrum at Zero Using a Bartlet window.
% Spec = bwspeczero(x,lt);
% Spec is the spectrum of X. Covz is the covariance matricies of X. 
% X is N obs by K variables matrix. 
% lt is the size of the window.  Cov(X(t+j),X(t)) will get zero weight for j > lt. 
% lamj is the values at which to evaluate the spectrum
% to get an estimate of the variance of the sample mean set lamj = 0 and use 2*pi*spec
% See Hamilton Chapt 10 p. 280-283


%create the window.
k=1:lt;
lam = 1-k/(lt+1); 
lamj = 0;


%get the size of the data and remove means
[maxt,nvars]= size(x);
x= x-repmat(mean(x),maxt,1);

%construct the cov matricies (Hamilton's cap Gamma k)
covz=zeros(nvars,nvars,lt);
for k=0:lt;
   covz(:,:,k+1) =(1/maxt)*x((k+1):maxt,:)'*x(1:maxt-k,:); 
end;

% %calculate the spectrum of the data Hamilton 10.4.42
% for zw = 1:length(lamj);
%    w=lamj(zw);
%    spec(:,:,zw)=(1/(2*pi))*covz(:,:,1);
%    for k=1:lt
%       spec(:,:,zw)=spec(:,:,zw)+(1/(2*pi))*(lam(k)*exp(-i*w*k)*squeeze(covz(:,:,k+1))+lam(k)*exp(i*w*k)*squeeze(covz(:,:,k+1))');
%    end;
% end;
%spec = squeeze(spec);
%cov = 2*pi*spec;   
 
zw = 1; 
   w=lamj(zw);
   altspec(:,:)= covz(:,:,1);
   for k=1:lt
      altspec(:,:)=altspec(:,:)+(lam(k)*squeeze(covz(:,:,k+1))+lam(k)*squeeze(covz(:,:,k+1))');
   end;

 cov = altspec; 