function [rr] = checkunit(macoef)
%
% this takes a pure moving average representation
%
%   y(t) = u(t) + macoef(1)*u(t-1) + ... + macoef(q)*u(t-q)
%
%  and computes the roots of the determinant of the moving
%  average, following the suggestion of Fernandez-Villaverde, Rubio-Ramirez and Sargent

[N,M]=size(macoef);
q=M/N;

if round(q)*N ~= M
    error('fatal (checkunit) something wrong with moving average coefficients')
end

if q == 1
    A=-macoef;
else
    A=zeros(q*N,q*N);
    for ii = 2:q
        A((ii-1)*N+[1:N],(ii-2)*N+[1:N])=eye(N);
    end
    A([1:N],[1:N*q])=-macoef;
end

rr=eig(A);