function [lam,Crl,Crh,Cul,Cuh,liker,likeu,Fr,Fu,per,omm,VARu,VARr,omega] = decompose(Y,sarcoef,macoef,v,q,p,ar,ma,vv,cut)
%
% This takes as input data, Y(:,1),...,Y(:,T), parameters of a restricted SARMA
% model, sarcoef, macoef, v, and parameters of an unrestricted SARMA model,
% ar, ma, vv. 
%
% On output it returns the periodogram, per, and the spectrum of 
% restricted model (Fr), and unrestricted model (Fu). In 
% addition, it returns the cumulative likelihood ratio statistic in 
% frequency domain.
% The variance of the restricted process is Crl+Crh, and the variance of
% the unrestricted process is Cul+Cuh. Here, l denotes frequencies whose
% periods are longer than cut and h denotes frequencies whose period are
% shorter than cut.
% The frequency domain things that are returned are
% for frequencies 2*pi*j/T, j=0,...,T/2.
%

i=sqrt(-1);

[m,T]=size(Y);

if round(T/2)*2 ~= T
    error('fatal (spectral) number of observations must be even')
end

liker=0;
likeu=0;

for j = 0:T/2
    
    omm(j+1)=2*pi*j/T;
    om=omm(j+1);
    
    a=zeros(m,1);
    for t=1:T
        a=a+Y(:,t)*exp(-i*om*t);
    end
    per(:,:,j+1)=a*a'/(2*pi*T);
    
    a=1;
    aa=1;
    for ii = 1:p
        a=a-sarcoef(ii)*exp(-i*om*ii);
        aa=aa-ar(ii)*exp(-i*om*ii);
    end
    
    b=eye(m);
    bb=eye(m);
    for ii = 1:q
        b=b+macoef(:,(ii-1)*m+[1:m])*exp(-i*om*ii);
        bb=bb+ma(:,(ii-1)*m+[1:m])*exp(-i*om*ii);
    end
    
    Fr([1:m],[1:m],j+1)=(1/(2*pi))*b*v*b'/(a*a');%restricted reduced form
    Fu([1:m],[1:m],j+1)=(1/(2*pi))*bb*vv*bb'/(aa*aa');%unrestricted reduced form       
    
    detFr=Fr(1,1,j+1)*Fr(2,2,j+1)-Fr(1,2,j+1)*Fr(2,1,j+1);
    detFu=Fu(1,1,j+1)*Fu(2,2,j+1)-Fu(1,2,j+1)*Fu(2,1,j+1);
    
    lamm=log(detFr/detFu)+trace( ( inv(Fr(:,:,j+1)) - inv(Fu(:,:,j+1)) ) * per(:,:,j+1) );
    if abs(imag(lamm)/real(lamm)) > 6e-5 
        error('fatal (spectral) complex likelihood ratio')
    end
    
    ispr=inv(Fr([1:m],[1:m],j+1));
    Lr=-( 2*log(2*pi) + log(detFr) + trace(ispr*per(:,:,j+1)) )/2;
    ispu=inv(Fu([1:m],[1:m],j+1));
    Lu=-( 2*log(2*pi) + log(detFu) + trace(ispu*per(:,:,j+1)) )/2;
    
    if j == 0
        lam(1)=real(lamm);
        liker=Lr;
        likeu=Lu;
    end
    if j > 0 & j < T/2
        lam(j+1)=lam(j)+2*real(lamm);
        liker=liker+2*Lr;
        likeu=likeu+2*Lu;
    end
    if j == T/2
        lam (j+1)=lam(j)+real(lamm);        
        liker=liker+Lr;
        likeu=likeu+Lu;
    end  
    
end

N=10000;
[Cul,Cuh,Crl,Crh,VARu,VARr,omega] = getvar(N,sarcoef,macoef,v,ar,ma,vv,p,q,cut);

if abs(imag(liker)) > .9e-10
    error('fatal (decompose) restricted likelihood function complex')
end
if abs(imag(likeu)) > .9e-10
    error('fatal (decompose) unrestricted likelihood function complex')
end

liker=real(liker);
likeu=real(likeu);