function [bbet,C,V,rrts,Y,Isum,Q,S,v,v1] = getmodelvar(T,N,p,lam,muz,l,k,c,y,theta,delta,h0,h1,A,B,P,sigz,sigl,sigx,J,f,lngrun,Tt)
%
%This routine takes the model solution output by solvemodel.m and computes
%the population bivariate VAR for productivity growth and log hours. The VAR 
%has p lags, and N points are used to do the Riemann approximation of the integral. Let 
%
%             Y(t) = [dlog(y(t)/l(t)) log(l(t))]'.
%
%Then, bbet = [A1,...,Ap], where
%
%           Y(t) = A1*Y(t-1) + A2*Y(t-2) + ... + Ap*Y(t-p) + u(t),
%           Eu(t)u(t)' = V.
%
%Also, rrts is the eigenvalues of the companion matrix formed from A1,...,Ap.
%The program outputs C, where
%  
%                               C*C' = V
%                          [I-sum(A1,...,Ap)](-1)*C, when Tt=inf, or Tt=[],
%                          or Tt is just missing from the input list. When Tt is finite, the
%                          restriction involves the forecast revision Tt
%                          periods ahead.
%
% has zeros in the first row, except in the 1,1 element which is chosen to be non-negative.
% If f is not [], then the data are first transformed to eliminate
% components with period f and less. If f=[], this is ignored.
%
% The program returns Y(1),....,Y(T), the response of the system to
% the long-run (technology) shock. This is identified in two different
% ways, depending on the setting of lngrun. If lngrun = 1, then long-run
% identification is done. If lngrun = 0, then recursive identification is
% done.
%
%   The model also allows the user to return the variances of HP-filtered
% data. The vector, v, has the same dimension as W, and contains the variance of
% HP-filtered, log output, hours, investment and consumption, in that order. 
% The user must, on input, provide lam.
%
% The vector, J, allows the user to select which variables appear in Y(t).
% Let: dlog(Y/l) be (1), log(l) be (2), log(I/Y) be (3) and log(C/Y) be (4).
% If J=[1 2] then the first two variables only are included. If J=[1 2 4],
% then the 4th is also included, etc.
% 

%first, get the representation of Y(t):
%
%                  Y(t) = a0 + a1*khat(t) + a2*khat(t-1) + b0*s(t) +  b1*s(t-1)
%
%where s(t) is defined in the writeup to solvemodel.m.
%

[a0,a1,a2,b0,b1,W] = reducedform(A,B,muz,l,k,c,y,delta,theta,h0,h1,sigz,sigl,sigx);

% if length(J) > rank(W)
%     error('fatal (getmodelvar) more variables than shocks in VAR')
% end

%go get the VAR asociated with the function passed through findAR.
sw=1;
[bbet,V,S,v,v1] = findAR(p,N,W,lam,J,f,sw,'dee',A,a1,a2,b0,b1,B,P);

%now the var reduced form matrices, bbet, and the error term
%variance-covariance matrix, V are used. The first column of C describes
%the response to a technology shock.
if lngrun == 1
    
    Bsum=[];
    TT=[];
    [C,Isum,Q] = longrun(bbet,V,Bsum,TT);
    
elseif lngrun == 0% in this case, do recursive identification....see the technical appendix for an explanation of these calculations
    
    Q=[];
    
    Isum=[];
    
    if size(V,1) == 2
        E=[0 1
            1 0];
    else
        E=[0 1 0
            1 0 0
            0 0 1];
    end
    Vt=E*V*E';
    Ct=chol(Vt)';
    C=[E'*Ct(:,2) zeros(size(Vt,1),size(Vt,2)-1)];
    
elseif lngrun == 2%factorize the true spectral density at frequency zero
    
    [n,m]=size(bbet);
    p=m/n;
    sum=eye(n);    
    for ii = 1:p
        sum=sum-bbet(:,(ii-1)*n+[1:n]);
    end
    Q=real(S(J,J,1));
    D=chol(Q)';
    C=sum*D;
    Isum=inv(sum);
    
end

n=size(C,1);

[rrts] = companion(n,p,bbet);

%simulate the response of the system to the technology shock...
[Y] = impVAR(T,C,bbet);