function [sarcoef,macoef,v,q,p] = inp(theta,muz,taux,beta,delta,gamma,taul,psi,lb,rhoz,rhox,rhol,tau,sigz,sigl,sigx,sig,rv,data,plag,qlag)
%provides starting values for estimation of a SARMA model.  
%when rv equals zero, the starting values come from solving an RBC model. 
%when rv equals one, the starting values come from first estimating a VAR
%with many lags to get the forecast error.  Then the estimated forecast
%errors are used as observed variables to estimate the SARMA. 
if rv == 0
    
    chk=0;%check all analytic derivatives used to solve the model, by evaluating the numerical analogs
    if abs(sig-1) < .1e-9
        
        [A,B,P,a0,a1,a2,b0,b1,W,y,l,c,k] = solution(theta,muz,taux,beta,delta,gamma, ...
            taul,psi,lb,rhoz,rhox,rhol,tau,sigz,sigl,sigx,sig,chk);
        
    else%if sig ~= 1, then compute the value of psi that rationalizes the steady state employment at sig = 1
        
        ssig=1;
        [A,B,P,a0,a1,a2,b0,b1,W,y,l,c,k] = solution(theta,muz,taux,beta,delta,gamma, ...
            taul,psi,lb,rhoz,rhox,rhol,tau,sigz,sigl,sigx,ssig,chk);
        ll=l;
        [A,B,P,a0,a1,a2,b0,b1,W,y,l,c,k,psi] = solution(theta,muz,taux,beta,delta,gamma, ...
            taul,psi,lb,rhoz,rhox,rhol,tau,sigz,sigl,sigx,sig,chk,ll);
        
    end 
    J=[1 2];
    II=eye(4,4);
    II=II(J,:);
    a0=II*a0;
    a1=II*a1;
    a2=II*a2;
    b0=II*b0(:,J);%right now, this is programmed to only handle the case of two shocks
    b1=II*b1(:,J);
    
    h0=A(2,1);
    h1=B(2,J);
    A=A(1,1);
    B=B(1,J);
    
    D1=zeros(2,2);
    D1(1,1)=-rhol;
    ib0=inv(b0);
    C0=eye(2);
    C1=((a1*B-b0*A+b1)+b0*D1)*ib0;
    C2=((a1*B-b0*A+b1)*D1+(a2*B-b1*A))*ib0;
    C3=(a2*B-b1*A)*D1*ib0;
    macoef=[C1 C2 C3];
    q=3;
    if max(max(abs(C3))) < .1e-10
        macoef=[C1 C2];
        q=2;
    end
    
    if abs(rhoz) < .1e-10
        p=2;
        sarcoef(1)=rhol+A;
        sarcoef(2)=-rhol*A;
    else
        p=3;
        sarcoef(1)=rhoz+rhol+A;
        sarcoef(2)=-(rhoz*rhol+(rhoz+rhol)*A);
        sarcoef(3)=rhoz*rhol*A;
    end
    
    W=[sigz^2 0
        0 sigl^2];
    
    v=b0*W*b0';
    
elseif rv == 1
    
    p=plag;
    q=qlag;
    [sarcoef,macoef,v] = stepone(data,plag,qlag);
    
end