function [F,H,W] = kalmanmat(h0,h1,A,B,theta,delta,gam,muz,k,c,y,l,sigz,sigl,sigx,P,option)

%This returns the state/observer representation of the DSGE model:
% S(t)=F*S(t-1)+eps(t), Eeps(t)*eps(t)'=W
% X(t)=H*S(t).
%option controls the structure of X(t), as explained in runSARMA.m

[yk,ys,ik,is,ck,cs] = rrform(h0,h1,A,B,theta,delta,muz,k,c,y,l,gam);
n=size(B,2);
tau=zeros(1,n);
tau(1)=1;

Q=zeros(n,n);
Q(1,1)=sigz;
Q(2,2)=sigl;
Q(3,3)=sigx;

F=zeros(2+2*n,2+2*n);
F(1,1)=A;
F(1,[2+1:2+n])=B;
F(2,1)=1;
F([2+1:2+n],[2+1:2+n])=P;
F([2+n+1:2+n+n],[2+1:2+n])=eye(n);

H=zeros(4,2+2*n);

H(1,1)=yk;
H(1,2)=-yk;
H(1,[2+1:2+n])=tau+ys;
H(1,[2+n+1:2+n+n])=-ys;

H(2,1)=h0;
H(2,[2+1:2+n])=h1;

if option == 1
    
    H(3,1)=ik-yk;
    H(3,[2+1:2+n])=is-ys;
    H(4,1)=ck-yk;
    H(4,[2+1:2+n])=cs-ys;
    
else
    
    H(3,1)=ik;
    H(3,2)=-ik;
    H(3,[2+1:2+n])=tau+is;
    H(3,[2+n+1:2+n+n])=-is;
    
    H(4,1)=ck;
    H(4,2)=-ck;
    H(4,[2+1:2+n])=tau+cs;
    H(4,[2+n+1:2+n+n])=-cs;
    
end

if option == 3
    H(1,1)=theta*(1-h0);
    H(1,2)=-H(1,1);
    H(1,[2+1:2+n])=((1-theta)*tau-theta*h1);
    H(1,[2+n+1:2+n+n])=theta*(tau+h1);
end

W=zeros(2+2*n,2+2*n);
W([2+1:2+n],[2+1:2+n])=Q*Q';
