function [C,Isum,Q] = longrun(bbet,V,Bsum,T)
%
%This computes the impact multiplier of the fundamental shocks given a var
%representation, with var lag coefficients bbet and variance-covariance
%coefficients, V.
%
%It is not necessary to include Bsum and T in call
%If you want T not zero, then Bsum must be in call to longrun, even if only
%as an empty object.%
%
[n,m]=size(bbet);
p=m/n;

if round(p) ~= p
    error('fatal (longrun) dimenstions of bbet not as expected')
end

if nargin < 4
    T=[];
end
if nargin < 3
    Bsum=[];
end

if max([isinf(T) == 1  isempty(T) == 1]) == 1
    

    sum=eye(n);

    for ii = 1:p
        sum=sum-bbet(:,(ii-1)*n+[1:n]);
    end

    Isum=inv(sum);

    if isempty(Bsum) == 0 
        Isum = inv(eye(n)-Bsum);
        sum = eye(n) - Bsum;
    end

    Q=Isum*V*Isum';
    D=chol(Q)';
    C=sum*D;

else

    [rrts,A] = companion(n,p,bbet);
    I=eye(size(A,1));
    BT=I;
    for ii = 1:T
        BT=A*BT+I;
    end
    BT1=BT([1:n],[1:n]);
    Q=BT1*V*BT1';
    D=chol(Q)';
    C=BT1\D;
    Isum=BT1;
    
end