%This code lets the user reproduce calculations underlying Figure 1 and Table 1 in the text.
%The variance decompositions in the text are actually computed in population, doing the 
%relevant complex integration

clear all
theta=0.33;
beta=0.98^.25;
delta=1-(1-.06)^.25;
psi=2.5;
gamma=1.01^.25 - 1;
taux=.3;
taul=0.242;
muz=1.016^.25-1;
sig=1;
rhoz=0;
two=3;%if 2, then two-shock specification; if 3, then three shock.
tau=ones(3,2);%that there are all ones in here means we are solving the nonrecursive version of the model

T=2000;
if two == 2
    %two-shock ckm specification
    rhol=.952;
    sigz=.0131;
    sigl=.0136;
    rhox=.98;
    sigx=.00;
else
    %three-shock ckm specification
    rhol=.952;
    sigz=.0131;
    sigl=.0136;
    rhox=.98;
    sigx=.0123;
end

%this program solves the two- or three-shock model and simulates it under
%two scenarios: in one only the technology shocks are active, and in the
%other all shocks are active
%s (tech, labor tax, capital tax), time series
%z (normalized beginning of period capita, log hours), time series
[sckm,zckm,s1ckm,z1ckm] = lowvariancee(theta,muz,taux,beta,delta,gamma,taul,psi,rhoz,rhox,rhol,tau,sigz,sigl,sigx,sig,T);

if two == 2
    %two-shock mle specification
    sigz=0.00953;
    sigl=0.0056;
    rhol=.986;
    sigx=0;
    rhox=0;
else
    %three-shock mle specification
    sigz=0.00968;
    sigl=0.00631;
    rhol=.9994;
    sigx=0.00963;
    rhox=0.9923;
end

[s,z,s1,z1] = lowvariancee(theta,muz,taux,beta,delta,gamma,taul,psi,rhoz,rhox,rhol,tau,sigz,sigl,sigx,sig,T);

vvckm1=var(z1ckm(2,:));
vvckm=var(zckm(2,:));
vv1=var(z1(2,:));
vv=var(z(2,:));

if two == 2
    fprintf('Percent of variance in log hours due to technology, two-shock ckm model = %6.3f\n',100*vvckm1/vvckm)
    fprintf('Percent of variance in log hours due to technology, two-shock mle model = %6.3f\n',100*vv1/vv)
else
    fprintf('Percent of variance in log hours due to technology, three-shock ckm model = %6.3f\n',100*vvckm1/vvckm)
    fprintf('Percent of variance in log hours due to technology, three-shock mle model = %6.3f\n',100*vv1/vv)
end

tt=1:200;
subplot(211)
plot(tt,z1(2,tt),tt,z(2,tt))
if two == 2
    title('Two-shock MLE model, log hours with both shocks and with just technology shocks')
else
    title('Three-shock MLE model, log hours with both shocks and with just technology shocks')
end

subplot(212)
plot(tt,z1ckm(2,tt),tt,zckm(2,tt))
if two == 2
    title('Two-shock CKM model, log hours with both shocks and with just technology shocks')
else
    title('Three-shock CKM model, log hours with both shocks and with just technology shocks')
end

