%this routine is used to estimate the DSGE model, and also to estimate the
%unrestricted SARMA reduced form corresponding to that model
%when datasource=0, then the nonlinear estimation routine for the SARMA is
%checked on artificial data. Output from some runs of this program appears
%in fitting.m

option=3;%option: if =0, then do X=[dlog(Y) log(labor) dlog(Inv) dlog(C)], 
                %= 1 then X=[dlog(Y) log(labor) log(Inv/Y) log(C/Y)]
                %= 3, then X=[dlog(Y/labor) log(labor) dlog(Inv) dlog(C)]
                
sht=3;%if = 1 or 2, then use only output growth and labor and set meas error=0,(if=1, cev data, =2, ckm data)
      % if = 0 use all four variables, cev data
      % if= 3, use the first three variables, estimate the parameters of
      % technology, preferences and capital taxes. No measurement error.
      % Use cev data
      
const = 0;
rv=0;%=1, use vigfusson's program to get the initial estimates of the sarma model
         %=0, use the model to get the initial estimates
datasource=1;%if =1, then use actual data, otherwise, generate random data using model parameters in paramsnew 
spectrum = 0;%if = 1, do all estimation using the frequency domain approximation to the spectrum...if = 0, do exact

setpath
paramsnew

tau=ones(3,2);%specifies that all decisions see all shocks...the rest of the code assumes this
         
[Y] = getdata(option,sht);

if const == 0%if no constant is to be used in the analysis, then take mean out of the data
    mm=mean(Y');
    [m,T]=size(Y);    
    for ii = 1:m
        Y(ii,:)=Y(ii,:)-ones(1,T)*mm(ii);
    end
end

data=Y';
N=size(data,2);

%
%the following code sets up the constrained estimation of the 
%dsge model, starting from the parameters in paramsnew (as possibility
%ammended in the next few lines)
%if maxiter=0, then the program just returns the function value, which 
%can then be compared with the unconstrained estimation results, in case
%actual data is being used.
%if artificial data are to be used, then the coefficients output from estimatemodel.m is
%used to generate an artificial data set below (see what happense when
%datasource=0)
%

stderr=1;
RR=zeros(2,2);
%sht=1;
const=0;
measerr=0;
maxiter=1000;

if sht == 2
    %estimation results with ckm data: fval = -1.221928622450855e+003
    rhol =  0.99109657538683;
    sigl = 0.00567405091800;
    sigz = 0.01133249514237;
elseif sht == 1
    %estimation results with cev data: fval=-1.190043755444684e+003
    rhol =  0.98627471417620;
    sigl = 0.00569627440595;    
    sigz = 0.00954057006633;
    if spectrum == 1 
        rhol =   0.98800310347395;
        sigl =  0.00577685270699;
        sigz =  0.00957495833237;
    end
end

%ckm's june baseline
%     rhol=.952;
%     sigz=.0131;
%     sigl=.0136;
%     muz=.00516;
%     taul=.243;
%     thet=.33;
%     phi=2.5;


[rhol,sigl,sigz,rhox,sigx,R,se,H,fval1,xxnew,Yout] = estimatemodel(Y,theta,muz,taux,beta,delta,gamma,taul,psi,lb,rhoz,rhox,sigx, ...
    RR,rhol,sigl,sigz,option,sht,const,stderr,measerr,maxiter,spectrum);

%find a set of initial parameter values (rv=1, use rob's method, rv=0 use
%the DSGE model whose parameters have been read in)
[sarcoef,macoef,v,q,p] = inp(theta,muz,taux,beta,delta,gamma,taul,psi,lb,rhoz,rhox,rhol,tau,sigz,sigl,sigx,sig,rv);

if datasource == 0       
    T=1000;
    v=v/100;
    [Y] = rnd(T,p,q,N,sarcoef,macoef,v);
    data=Y;
end
 
sarcoeftrue=sarcoef;
macoeftrue=macoef;
vtrue=v;

maxiter=5000;
sarcoef=[];%if this is empty, but is included in the input list of estimateSARMA, this means an initial guess has been input
                  %manually in estimateSARMA
[fval,sarcoefnew,macoefnew,vnew,fvalnew,xxnew] = estimateSARMA(data,p,q,maxiter,spectrum,sarcoef,macoef,v);

cc=zeros(1,p+1);
cc(1)=1;
for ii = 2:p+1
    cc(ii)=-sarcoefnew(ii-1);
end

rr=roots(cc);

if p == 2 & imag(rr(1)) == 0
    sprintf(' ar roots (stationarity requires they lie inside the unit circle) = %7.5f, %7.5f\n',rr) 
elseif p == 3 & max(abs(imag(rr))) < .1e-10
    sprintf(' ar roots (stationarity requires they lie inside the unit circle) = %7.5f, %7.5f, %7.5f\n',rr) 
elseif p == 2 & abs(imag(rr(1))) > .1e-10
    sprintf(' ar roots (stationarity requires they lie inside the unit circle) = %9.7f, %9.7f\n',real(rr(1)),imag(rr(1)))
    sprintf(' ar roots (stationarity requires they lie inside the unit circle) = %9.7f, %9.7f\n',real(rr(2)),imag(rr(2)))
end

A=[-macoefnew(:,[1:2]) -macoefnew(:,[3:4])
    eye(2) zeros(2,2)];
r=eig(A);