%This program simulates the small sample distribution of the impulse
%response function from a VAR.
%close all
clear all
close all
setpath

%load up the parameter values:
paramsnew
def=2;%this determines the definition of X(t) in constructing a0, a1, a2, b0, b1, see rform.m

tau=[1 0
         1 1
         1  0];%specifies that investment responds to everything and labor only to second shock.
     
tau=ones(3,2);%specifies that all decisions see all shocks...the rest of the code assumes this

chk=0;%check all analytic derivatives used to solve the model, by evaluating the numerical analogs

%first, solve the model. Treat the case where sig=1 differently than sig.ne.1
if abs(sig-1) < .1e-9
    
    [A,B,P,a0,a1,a2,b0,b1,W,y,l,c,k] = solution(theta,muz,taux,beta,delta,gamma, ...
        taul,psi,lb,rhoz,rhox,rhol,tau,sigz,sigl,sigx,sig,chk,def);
    
else%if sig ~= 1, then compute the value of psi that rationalizes the steady state employment at sig = 1
    
    ssig=1;
    [A,B,P,a0,a1,a2,b0,b1,W,y,l,c,k] = solution(theta,muz,taux,beta,delta,gamma, ...
        taul,psi,lb,rhoz,rhox,rhol,tau,sigz,sigl,sigx,ssig,chk,def);
    ll=l;
    [A,B,P,a0,a1,a2,b0,b1,W,y,l,c,k,psi] = solution(theta,muz,taux,beta,delta,gamma, ...
        taul,psi,lb,rhoz,rhox,rhol,tau,sigz,sigl,sigx,sig,chk,def,ll);
    
end 

h0=A(2,1);
h1=B(2,:);
A=A(1,1);
B=B(1,:);

if length(J) > rank(W) & par ~= 5 & par ~= 8
    error('fatal (getmodelvar) more variables than shocks in VAR')
end

%get parameters of VAR(infinity) using Proposition 1 formula
TT=50;
[M,varcoef,C,Bsum] = sargentvillaverde(theta,muz,taux,beta,delta,gamma,taul,psi,lb,rhoz,rhox,rhol,sigz,sigl,sigx,P,sig,tau,J,TT,def);

%Following lines contain various checks
%spectral density at frequency zero implied by VAR(infinity)
SSS0=inv(eye(length(J))-Bsum)*(C*C')*inv(eye(length(J))-Bsum');

%spectral density at frequency zero computed from pure moving average
%representation, dee:
D=feval('dee',1,A,a1,a2,b0,b1,B,P); 
S0=D(J,:)*W*D(J,:)';

%compare the spectral density at frequency zero implied by the
%VAR(infinity) representation and the spectral density at frequency zero
%implied by dee....
if max(max(abs(S0-SSS0))) > .1e-10
    error('fatal (smallsample) error in spectral density at frequency zero')
end

%get the lead matrix in pure moving average representation:
D0=feval('dee',0,A,a1,a2,b0,b1,B,P); 
JD0=D0(J,J);
JD=D(J,J);
IMB1=JD0*inv(JD);
V=JD0*W(J,J)*JD0';

%compare the one-step-ahead forecast error variance implied by the
%var(infinity) with the one-step-ahead forecast error variance in pure ma, dee
if max(max(abs(V-C*C'))) > .1e-10
    error('fatal (smallsample) one-step-ahead forecast error variance in var(infinity) is incorrect')
end

SS0=inv(IMB1)*V*inv(IMB1)';
if max(max(abs(SS0-S0))) > .1e-11
    error('fatal (smallsample) problem occured')
end

C1=D0*sqrt(W);
N=1000;
lam=1600;
f=[];
p=4;
sw=0;
%compute the plim of the p-lag VAR, as well as various population second moment statistics for the model
[bbet,V,S,v,v1] = findAR(p,N,W,lam,J,f,sw,'dee',A,a1,a2,b0,b1,B,P);
sY=100*sqrt(v(1));
sprintf('percent contemporaneous impact on hours of one-standard deviation shock to technology %6.5f\n',100*D0(2,1)*sqrt(W(1,1)))
sprintf('percent of hp-filtered variance in output due to technology = %6.5f\n',100*v1(1)/v(1))
sprintf('percent variance in log hours due to technology = %6.5f\n',100*v1(2)/v(2))
percent1=100*D0(1,1)*W(1,1)/(D0(1,1)*W(1,1)+D0(1,2)*W(2,2)+D0(1,3)*W(3,3));
percent2=100*D0(1,2)*W(2,2)/(D0(1,1)*W(1,1)+D0(1,2)*W(2,2)+D0(1,3)*W(3,3));
percent3=100*D0(1,3)*W(3,3)/(D0(1,1)*W(1,1)+D0(1,2)*W(2,2)+D0(1,3)*W(3,3));

if abs(sig) < .1e-10
    frisch=Inf;
else
    frisch=(lb-l)/(sig*l);
end

%Now, go to work on the small sample (T observations) distribution of VAR estimate of 
%response of hours worked to a technology shock, when a p-lag VAR is estimated.
%The number of impulse response lags to compute is Timp.
%The number of artificial samples to use is Tsim

p=4;
T=180;
Timp=20;
Imp=zeros(1,Timp);
Impckm=zeros(1,Timp);
Imp2=zeros(1,Timp);
Impckm2=zeros(1,Timp);
Tsim=1000;
if dtrue == 1
    Tsim=1;
end
const=1;
icount=0;
n=length(J);

if err > .1e-10
    n = n+1;
end

Imp=zeros(n,Timp);
Impckm=zeros(n,Timp);
Imp2=zeros(n,Timp);
Impckm2=zeros(n,Timp);
MS0hat=zeros(n);
MS0hat2=zeros(n);

for ii = 1:Tsim
    %if Bsum is not empty and var=1 in paramsnew, then use the true B(1) in computing the impulse response functions
    Bs=Bsum; 
    Bs=[];
    [imp,impckm,lhat,rrts,C,Cckm,Isum,data,bbet,V,n,cond,S0hat,qstat,techf] = varimp(T,Timp,p,W, ...
        sigz,A,B,P,h0,h1,a0,a1,a2,b0,b1,const,J,err,S0,dtrue,C1,var,Bs,S0);
    if isempty(techf) ~= 1;percentz(ii)=techf;end
    if isempty(rrts) ~= 1;rts(ii)=max(rrts);end
    
    qstatistic(ii)=qstat;
    condition(ii)=cond;
    MS0hat=MS0hat+S0hat;
    MS0hat2=MS0hat2+S0hat.^2;
    Imp=Imp+imp;
    Impckm=Impckm+impckm;
    Imp2=Imp2+imp.^2;
    Impckm2=Impckm2+impckm.^2;
    DD=Isum*C;
    if size(DD,1) > 2
        DDckm=zeros(length(J),3);
    else
        DDckm=Isum*Cckm;
        if sign(DD(1,1)) ~= sign(DDckm(1,1))
            icount=icount+1;
            prt=0;
            if prt == 1
            subplot(221)
            plot(100*cumsum(imp(1,:)))
            title('productivity (LR)')
            subplot(223)
            plot(100*imp(2,:))
            title('hours (LR)')
            subplot(222)
            plot(100*cumsum(impckm(1,:)))
            title('productivity')
            subplot(224)
            plot(100*impckm(2,:))
            title('hours')
            pause
            end
        end            
    end
    
    [rrts] = companion(n,p,bbet);
    YY(ii,1)=DD(1,1);
    YY(ii,2)=100*C(2,1);
    YY(ii,[3:length(rrts)+2])=abs(rrts)';
    if max(abs(rrts)) > 1
        iss=1;
    else
        iss=0;
    end
    YY(ii,length(rrts)+3)=iss;
    YY(ii,length(rrts)+4)=100*Cckm(2,1);
end
Imp=Imp/Tsim;
Impckm=Impckm/Tsim;
Imp2=Imp2/Tsim - Imp.^2;
Impckm2=Impckm2/Tsim - Impckm.^2;
Imp2=100*sqrt(Imp2);
Impckm2=100*sqrt(Impckm2);
Imp=100*Imp;  
Impckm=100*Impckm;  
MS0hat=MS0hat/Tsim;
sS0hat=sqrt((MS0hat2/Tsim)-MS0hat.^2);

fct=2;
vbl=2;
if smple == 1
    tt=1:Timp;
    plot(tt,Imp(vbl,:),tt,100*lhat,'*',tt,Imp(vbl,:)+fct*Imp2(vbl,:),'--',tt,Imp(vbl,:)-fct*Imp2(vbl,:),'--',tt,100*lhat)
    legend('Mean, Small Sample VAR','True','Plus, Minus 2 Standard Deviations')
    if par == 1
        title('Bivariate VAR, Kydland-Prescott Model')   
        ylabel('Percent Response in Hours to One-Standard Deviation (1.17 Percent) Technology Shock')
    end
    if par == 2
        title('Bivariate VAR, CKM Benchmark')   
        ylabel('Percent Response in Hours to One-Standard (0.58 Percent) Deviation Technology Shock')
    end
    if par == 3
        title('Trivariate VAR, Three Shocks, Two Important (CKM)')   
        ylabel('Percent Response in Hours to One-Standard Deviation Technology Shock')
    end
    if par == 4
        title('Trivariate VAR, Three Important Shocks (CKM)')   
        ylabel('Percent Response in Hours to One-Standard Deviation Technology Shock')
    end
    if par == 5
        title('Four Variable VAR, Four Shocks, Three Important (CKM)')   
        ylabel('Percent Response in Hours to One-Standard Deviation Technology Shock')
    end
    if par == 6
        title('Trivariate VAR, Three Shocks, Two Important (KP)')   
        ylabel('Percent Response in Hours to One-Standard Deviation Technology Shock')
    end
    if par == 7
        title('Trivariate VAR, Three Important Shocks (KP)')   
        ylabel('Percent Response in Hours to One-Standard Deviation Technology Shock')
    end
    if par == 8
        title('Four Variable VAR, Four Shocks, Three Important (KP)')   
        ylabel('Percent Response in Hours to One-Standard Deviation Technology Shock')
    end
    xlabel('Quarters')
    sprintf('mean, impact on hours = %4.2f std dev = %4.2f, Monte Carlo std dev %4.2f\n',Imp(vbl,1),Imp2(vbl,1),Imp2(vbl,1)/sqrt(Tsim))
    sprintf('CKM identification, mean hours response = %4.2f, std dev = %4.2f, Monte Carlo %4.2f\n',Impckm(vbl,1),Impckm2(vbl,1),Impckm2(vbl,1)/sqrt(Tsim))
 
    return
end

n=length(J);
if n == 2
    subplot(221)
else
    subplot(211)
end
tt=1:Timp;
plot(tt,Imp(vbl,:),tt,Imp(vbl,:)+fct*Imp2(vbl,:),tt,Imp(vbl,:)-fct*Imp2(vbl,:),tt,100*lhat,'*',tt,100*lhat)
title('Long-run Identification')
%axis([0 20 -1.5 1.5])
if n == 2
    subplot(223)
else
    subplot(212)
end
hist(YY(:,2))
%axis([-1.5 1.5 0 300])
if length(J) > 2
    if err > .1e-10
        str=['Four Variate \tau_x = ',num2str(taux),', \tau_l = ',num2str(taul),', \mu_z = ',num2str(muz), ...
            ', \rho_l = ',num2str(rhol),', \sigma_z = ',num2str(sigz),', \sigma_l = ',num2str(sigl), ...
            ', \rho_z = ',num2str(rhoz),', \rho_x = ',num2str(rhox),', \sigma_x = ',num2str(sigx),' err = ',num2str(err)];
    else
        str=['Trivariate \tau_x = ',num2str(taux),', \tau_l = ',num2str(taul),', \mu_z = ',num2str(muz), ...
            ', \rho_l = ',num2str(rhol),', \sigma_z = ',num2str(sigz),', \sigma_l = ',num2str(sigl), ...
            ', \rho_z = ',num2str(rhoz),', \rho_x = ',num2str(rhox),', \sigma_x = ',num2str(sigx)];
    end
    hout=suptitle(str);
    return
end
subplot(224)
hist(YY(:,length(rrts)+4))
%axis([-1.5 1.5 0 300])
subplot(222)
plot(tt,Impckm(vbl,:),tt,Impckm(vbl,:)+2*Impckmvbl(2,:),tt,Impckm(2,:)-vbl*Impckm2(vbl,:),tt,100*lhat,'*',tt,100*lhat)
title('CKM Identification')

str=['Bivariate \tau_x = ',num2str(taux),', \tau_l = ',num2str(taul),', \mu_z = ',num2str(muz), ...
   ', \rho_l = ',num2str(rhol),', \sigma_z = ',num2str(sigz),', \sigma_l = ',num2str(sigl), ...
   ', \rho_z = ',num2str(rhoz),', \rho_x = ',num2str(rhox),', \sigma_x = ',num2str(sigx)]; 
str=['Impact of CKM Identification on Sampling Uncertainty']
hout=suptitle(str);