function [H,F,D,Qt,n] = stateobserver(theta,muz,taux,beta,delta,gamma,taul,psi,lb,rhoz,rhox,rhol,tau,sigz,sigl,sigx,sig,J,JJ,def)

%
%Putting together a State-Observer representation. State: ki(t), Observer: x(t)
%x(t) elements of X(t) you are working with, with max being 3, and must
%include first and second.
%ki(t) = [kap(t) kap(t-1) s(t) s(t-1)], kap(t) - beginning of period
%capital stock, in percent deviation from steady state, s(t) - economic shocks
%s(t) = P*s(t-1) + Qt*e(t), e(t) - fundamental economic shocks
%ki(t) = F*ki(t-1) + D*e(t)
%x(t) = H*ki(t)
%

chk=0;%check all analytic derivatives used to solve the model, by
      % evaluating the numerical analogs
[A,B,P,a0,a1,a2,b0,b1,W,y,l,c,k] = solution(theta,muz,taux,beta,delta,gamma, ...
    taul,psi,lb,rhoz,rhox,rhol,tau,sigz,sigl,sigx,sig,chk,def);

h0=A(2,1);
h1=B(2,:);
A=A(1,1);
B=B(1,:);

n=size(B,2);

ttau=[1 0
    1 1
    1 0];

info=1;%info=1 means full information case

if max(max(abs(ttau-tau))) < .1e-10
    info=0;
end

if info == 0
    
    if n < 4 | n > 6
        error('fatal (sargentvillaverde) something wrong, unexpected dimension for B')
    end
    
else

    if n < 2 | n > 3
        error('fatal (sargentvillaverde) something wrong, unexpected dimension for B')
    end
    
end

Q=zeros(length(JJ),length(JJ));
Q(1,1)=sigz;
Q(2,2)=sigl;

if length(JJ) == 3;Q(3,3)=sigx;end

if length(JJ) < 3 & JJ(2) == 3
    Q(2,2)=sigx;
end

if info == 1
    n=length(JJ);
    Qt=Q;
    if n < 3;P=P(JJ,JJ);end
else
    Qt=zeros(2*length(JJ),length(JJ));
    Qt([1:length(JJ)],:)=Q;
    JJ=[JJ JJ+3];
    n=length(JJ);
    if n < 6;P=P(JJ,JJ);end
end

[n1,n2]=size(Qt);

if n1 ~= n
    error('fatal (sargentvillaverde) something wrong with left dimension of Qt')
end

if n2 ~= size(Q,2)
    error('fatal (sargentvillaverde) something wrong with right dimension of Qt')
end


H=[a1(J,:) a2(J,:) b0(J,JJ) b1(J,JJ)];
F=zeros(2+2*n,2+2*n);
F(1,1)=A;
F(1,2+[1:n])=B(JJ);
F(2,1)=1;
F(2+[1:n],2+[1:n])=P;
F(2+n+[1:n],2+[1:n])=eye(n);
D=zeros(2+2*n,n2);
D(2+[1:n],[1:n2])=Qt;