function [imp,impckm,lhat,rrts,C,Cckm,Isum,data,bbet,V,n,cond,S0,qstat,techf] = ...
    varimp(T,Timp,p,W,sigz,A,B,P,h0,h1,a0,a1,a2,b0,b1,const,J,err,S0true,dtrue,C1,var,Bsum,SS0)
%
% This the model solution as input, generates T observations using
% a random number generator and simulates data on productivity growth
% and log-hours and other variables, depending on the setting of J. 
% It then fits a VAR to these data and computes
% Timp impulse responses of productivity and hours to a technology shock, 
% identified using long-run restrictions. These appear in
%
%        imp(1),...,imp(Timp)
%
%        imp(t) = [d(log productivity)  log(hours)]'
%
% The true response of hours worked is in
%
%             lhat(1),...,lhat(Timp)
%
% const if unity, then include a constant in the simulated data and the
% var. if 0, then set the constant to zero in the simulated data and drop
% it in the var.
% 
% rrts - eigenvalues of Estimated VAR coefficients.
% cond - absolute value of eigenvaules of spect. density at frequency zero computed, cond=min/max eigenvalue.
% C - 2 by 2 matrix, with first column corresponding to impact effect of shock
% Isum - inverse([I-B(1)]), where B(1) is sum of estimated VAR matrices.
% Bsum - the B(1) in Isum is replaced by Bsum in case Bsum is not empty and var = 1
% data - T by 2 simulated data set, with first column being dlog(prod), second, log(hours).
% bbet - [A1,...,Ap] estimated VAR matrices
% V    - variance-covariance matrix of VAR disturbances, in simulation.
% S0 - true spectrum at frequency zero, used if var = 6.
%

epsil=sqrt(W)*randn(size(W,1),T);
[khat,lhat,s] = simulmodel(epsil,A,B,P,h0,h1,T);

if const==0
    a0=zeros(4,1);
end

X(:,1)=a0+b0*s(:,1);
X(:,2)=a0+a1*khat(2)+b0*s(:,2)+b1*s(:,1);

for ii = 3:T
    X(:,ii)=a0+a1*khat(ii)+a2*khat(ii-1)+b0*s(:,ii)+b1*s(:,ii-1);
end

if abs(err) > .1e-10
    eer=randn(T,1)*err;%including the price of investment, with measurement error
    data=[X(J,:)' s(3,:)'+eer];
else
    data=X(J,:)';
end

% If you uncomment the following lines, then the band pass filter is used
% to smooth the data a little prior to estimation.
% pl=2;
% pu=5;
% [fX] = bpass(data,pl,pu);
% data = data - fX;

if dtrue == 1
    load aceldata vardata
    if length(J) == 2
        JJ=[2 5];
    else
        JJ=[2:10 ];
    end
    data=vardata(:,JJ);
end

if const==0
    hasconstant=0;
else
    hasconstant=1;
end

[beta,V,uz,vcovbeta]=estimatevar(data,p,hasconstant);
s=6;
nlags=p;
test=1;
[qstat,dof,pvalue] = mkqmv(uz,s,nlags,test);
% ' here is the det of V'
% det(V)
% 'here is the condition number on V'
% cond(V)

if const == 1
    bbet=beta(:,[2:end]);
else
    bbet=beta(:,[1:end]);
end

lrun=0;
if lrun == 1
    [C,Isum] = longrun(bbet,V);
    S0=zeros(size(C,1));
    cond=.1;
else
    [C,Isum,S0,rrts] = longrunfreq0(bbet,V,data,uz,var,Bsum,SS0);    
    aa=abs(eig(S0));
    cond=min(aa)/max(aa);
end

itech=0;
if itech == 1
    sw=0;
    lam=1600;
    W=eye(size(V));
    f=[];
    N=1000;
    pp=p;
    [zt,zx,S,v,v1] = findAR(p,N,W,lam,J,f,sw,'dd',bbet,C,pp); 
    techf=v1(1)/v(1);
else
    techf=[];
end

[Cckm,Isumckm] = longrunckm(bbet,V);

n=size(data,2);
T=Timp;

%C=C1(J,J);%uncomment this for debugging purposes
[imp] = impVAR(T,C,bbet);
imp(1,:)=cumsum(imp(1,:));
if n <= 2
    [impckm] = impVAR(T,Cckm,bbet);
    impckm(1,:)=cumsum(impckm(1,:));
else
    impckm=zeros(n,T);
end

%model impulse responses
epsil=zeros(size(W,1),T);
epsil(1,1)=sigz;
[khat,lhat,s] = simulmodel(epsil,A,B,P,h0,h1,T);